// TestNonConstInitializer.cpp
// 
// Checks for a compiler bug where in certain cases a structure that should
// be allocated on the stack is allocated with static memory.
// This then causes problems if the function is re-entrant.

#include <stdio.h>

/* Output (with bug present)
[C64XP_0] Running with literal initializer:
[C64XP_0] level 0: initStruct.count:0
[C64XP_0] level 1: initStruct.count:1
[C64XP_0] level 2: initStruct.count:2
[C64XP_0] level 3: initStruct.count:3
[C64XP_0] level 4: initStruct.count:4
[C64XP_0] level 5: initStruct.count:5
[C64XP_0] Running with variable initializer:
[C64XP_0] level 0: initStruct.count:0
[C64XP_0] level 1: initStruct.count:0
[C64XP_0] level 2: initStruct.count:0
[C64XP_0] level 3: initStruct.count:0
[C64XP_0] level 4: initStruct.count:0
[C64XP_0] level 5: initStruct.count:0
*/

typedef struct  
{
  int unused1;
  int count;
} TEST_STRUCT;

int initValue = 42;

void TestStructure1(int count)
{
  TEST_STRUCT s = {42};
  s.count = count;

  if (count > 0)
  {
    TestStructure1(count - 1);
  }
  printf("level %d: initStruct.count:%d\n", count, s.count); 
}

void TestStructure2(int count)
{
  TEST_STRUCT s = {initValue};
  s.count = count;

  if (count > 0)
  {
    TestStructure2(count - 1);
  }
  printf("level %d: initStruct.count:%d\n", count, s.count); 
}

int main()
{
  printf("Running with literal initializer:\n");
  TestStructure1(5);
  printf("Running with variable initializer:\n");
  TestStructure2(5);

  return 0;
}
